@@ -52,8 +52,7 @@ static void go_component_set_property (GObject *obj, guint param_id,
 
 	switch (param_id) {
 	case COMPONENT_PROP_MIME_TYPE:
-		if (component->mime_type)
-			g_free (component->mime_type);
+		g_free (component->mime_type);
 		str = g_value_get_string (value);
 		if (str == NULL)
 			break;
@@ -107,8 +106,7 @@ go_component_finalize (GObject *obj)
 {
 	GOComponent *component = GO_COMPONENT (obj);
 
-	if (component->mime_type != NULL)
-		g_free (component->mime_type);
+	g_free (component->mime_type);
 
 	(*component_parent_klass->finalize) (obj);
 }
@@ -271,28 +269,16 @@ go_component_set_size (GOComponent *component, double width, double height)
 }
 
 void
-go_component_draw_cairo (GOComponent *component, gpointer data,
-												double width, double height)
+go_component_render (GOComponent *component, cairo_t *cr,
+			double width, double height)
 {
 	GOComponentClass *klass;
 
 	g_return_if_fail (IS_GO_COMPONENT (component));
 
 	klass = GO_COMPONENT_GET_CLASS(component);
-	if (klass->draw_cairo)
-		klass->draw_cairo (component, data, width, height);
-}
-
-void
-go_component_draw (GOComponent *component, int width_pixels, int height_pixels)
-{
-	GOComponentClass *klass;
-
-	g_return_if_fail (IS_GO_COMPONENT (component));
-
-	klass = GO_COMPONENT_GET_CLASS(component);
-	if (klass->draw)
-		klass->draw (component, width_pixels, height_pixels);
+	if (klass->render)
+		klass->render (component, cr, width, height);
 }
 
 gboolean
@@ -322,19 +308,6 @@ go_component_edit (GOComponent *component)
 	return FALSE;
 }
 
-void
-go_component_print (GOComponent *component, GnomePrintContext *gpc,
-												double width, double height)
-{
-	GOComponentClass *klass;
-
-	g_return_if_fail (IS_GO_COMPONENT (component));
-
-	klass = GO_COMPONENT_GET_CLASS(component);
-	if (klass->print)
-		klass->print (component, gpc, width, height);
-}
-
 void
 go_component_emit_changed (GOComponent *component)
 {
@@ -344,19 +317,6 @@ go_component_emit_changed (GOComponent *component)
 		go_component_signals [CHANGED], 0);
 }
 
-char *
-go_component_export_to_svg (GOComponent *component)
-{
-	GOComponentClass *klass;
-
-	g_return_val_if_fail (IS_GO_COMPONENT (component), NULL);
-
-	klass = GO_COMPONENT_GET_CLASS(component);
-	if (klass->export_to_svg)
-		return klass->export_to_svg (component);
-	return NULL;
-}
-
 static GOCmdContext *goc_cc;
 
 void
