@@ -21,6 +21,7 @@
 
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-chart-impl.h>
+#include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-plot-impl.h>
 #include <goffice/graph/gog-graph-impl.h>
 #include <goffice/graph/gog-style.h>
@@ -80,347 +81,6 @@ gog_axis_set_from_str (char const *str)
 
 /*****************************************************************************/
 
-struct _GogChartMap {
-	GogChart 		*chart;
-	GogViewAllocation	 area;
-	gpointer	 	 data;
-	GogAxisMap		*axis_map[3];
-	gboolean		 is_valid;
-
-	void (*map_2D_to_view)	(GogChartMap *map, double x, double y, double *u, double *v);
-};
-
-static void
-calc_polygon_parameters (GogViewAllocation const *area, GogChartMapPolarData *data, gboolean fill_area)
-{
-	double edges = data->th1 - data->th0 + 1.0;
-	double width = 2.0 * sin (2.0 * M_PI * go_rint (edges / 4.0) / edges);
-	double height = 1.0 - cos (2.0 * M_PI * go_rint (edges / 2.0) / edges);
-
-	data->rx = area->w / width;
-	data->ry = area->h / height;
-
-	if (!fill_area) {
-		data->rx = MIN (data->rx, data->ry);
-		data->ry = data->rx;
-	}
-
-	data->cx = area->x + area->w / 2.0;
-	data->cy = area->y + data->ry + (area->h - data->ry * height) / 2.0;
-}
-
-static void
-calc_circle_parameters (GogViewAllocation const *area, GogChartMapPolarData *data, gboolean fill_area)
-{
-	double x_min, x_max, y_min, y_max;
-	
-	if (data->th0 >= data->th1) {
-		x_min = y_min = -1.0;
-		x_max = y_max = 1.0;
-	} else {
-		double x;
-
-		if (data->th0 < -2.0 * M_PI) {
-			x = data->th0 - fmod (data->th0, 2.0 * M_PI);
-			data->th0 -= x;
-			data->th1 -= x;
-		} else if (data->th1 > 2.0 * M_PI) {
-			x = data->th1 - fmod (data->th1, 2.0 * M_PI);
-			data->th0 -= x;
-			data->th1 -= x;
-		}
-		if (data->th1 - data->th0 > go_add_epsilon (2 * M_PI)) 
-			data->th1 = data->th0 + 2 * M_PI; 
-
-		x_min = x_max = y_min = y_max = 0;
-		x = cos (-data->th0);
-		x_min = MIN (x_min, x); x_max = MAX (x_max, x);
-		x = sin (-data->th0);
-		y_min = MIN (y_min, x); y_max = MAX (y_max, x);
-		x = cos (-data->th1);
-		x_min = MIN (x_min, x); x_max = MAX (x_max, x);
-		x = sin (-data->th1);
-		y_min = MIN (y_min, x); y_max = MAX (y_max, x);
-
-		if (0 > data->th0 && 0 < data->th1)
-			x_max = 1.0;
-		if ((       M_PI / 2.0 > data->th0 &&        M_PI / 2.0 < data->th1) ||
-		    (-3.0 * M_PI / 2.0 > data->th0 && -3.0 * M_PI / 2.0 < data->th1))
-			y_min = -1.0;
-		if (( M_PI > data->th0 &&  M_PI < data->th1) ||
-		    (-M_PI > data->th0 && -M_PI < data->th1))
-			x_min = -1.0;
-		if (( 3.0 * M_PI / 2.0 > data->th0 &&  3.0 * M_PI / 2.0 < data->th1) ||
-		    (-      M_PI / 2.0 > data->th0 && -      M_PI / 2.0 < data->th1))
-			y_max = 1.0;
-	}
-	data->rx = area->w / (x_max - x_min);
-	data->ry = area->h / (y_max - y_min);
-	if (!fill_area) {
-		data->rx = MIN (data->rx, data->ry);
-		data->ry = data->rx;
-	}	
-	data->cx = -x_min * data->rx + area->x + (area->w - data->rx * (x_max - x_min)) / 2.0;
-	data->cy = -y_min * data->ry + area->y + (area->h - data->ry * (y_max - y_min)) / 2.0;
-}
-
-static void 
-null_map_2D (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	g_warning ("[GogChartMap::map_2D] not implemented");
-}
-
-typedef struct {
-	double a, b;
-} XMapData;
-
-static void 
-x_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	XMapData *data = map->data;
-
-	*u = gog_axis_map_to_view (map->axis_map[0], x);
-	*v = data->a * y + data->b;
-}
-
-typedef struct {
-	double		a[2][2];
-	double		b[2];
-} XYMapData;
-
-static void
-xy_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	*u = gog_axis_map_to_view (map->axis_map[0], x);
-	*v = gog_axis_map_to_view (map->axis_map[1], y);
-}
-
-static void
-polar_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	GogChartMapPolarData *data = (GogChartMapPolarData *) map->data;
-	double r = gog_axis_map_to_view (map->axis_map[1], y);
-	double t = gog_axis_map_to_view (map->axis_map[0], x);
-	
-	*u = data->cx + r * data->rx * cos (t);
-	*v = data->cy + r * data->ry * sin (t);		
-}
-
-/**
- * gog_chart_map_get_polar_parms:
- * @map: a #GogChartMap
- *
- * Convenience function for retrieving data related to polar plot layout.
- *
- * returns: a #GogChartMapPolarData struct.
- **/
-
-GogChartMapPolarData *
-gog_chart_map_get_polar_parms (GogChartMap *map)
-{
-	return (GogChartMapPolarData *) map->data;
-}	
-
-/**
- * gog_chart_map_new:
- * @chart: a #GogChart
- * @area: area allocated to chart
- * @axis0: 1st dimension axis
- * @axis1: 2nd dimension axis
- * @axis2: 3rd dimension axis
- * @fill_area: does chart fill allocated area
- *
- * Creates a new #GogChartMap, used for conversion from data space 
- * to canvas space.
- *
- * returns: a new #GogChart object. 
- **/
-
-GogChartMap *
-gog_chart_map_new (GogChart *chart, GogViewAllocation const *area, 
-		   GogAxis *axis0, GogAxis *axis1, GogAxis *axis2,
-		   gboolean fill_area)
-{
-	GogChartMap *map;
-	GogAxisSet axis_set;
-
-	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
-
-	map = g_new (GogChartMap, 1);
-
-	g_object_ref (chart);
-	map->chart = chart;
-	map->area = *area;
-	map->data = NULL;
-	map->is_valid = FALSE;
-
-	axis_set = gog_chart_get_axis_set (chart);
-	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
-		case GOG_AXIS_SET_X:
-			{
-				XMapData *data = g_new (XMapData, 1);
-
-				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
-				map->axis_map[1] = map->axis_map[2] = NULL;
-
-				data->b = area->y + area->h;
-				data->a = - area->h;
-
-				map->map_2D_to_view = x_map_2D_to_view;
-				map->data = data;
-
-				map->is_valid = gog_axis_map_is_valid (map->axis_map [0]);
-				break;
-			}
-		case GOG_AXIS_SET_XY:
-		case GOG_AXIS_SET_XY_pseudo_3d:
-		case GOG_AXIS_SET_XY_COLOR:
-			{
-				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
-				map->axis_map[1] = gog_axis_map_new (axis1, map->area.y + map->area.h, 
-								     -map->area.h);
-				map->axis_map[2] = NULL;
-
-				map->data = NULL;
-				map->map_2D_to_view = xy_map_2D_to_view;
-
-				map->is_valid = gog_axis_map_is_valid (map->axis_map[0]) &&
-					gog_axis_map_is_valid (map->axis_map[1]);
-				break;
-			}
-		case GOG_AXIS_SET_RADAR:
-			{
-				GogChartMapPolarData *data = g_new (GogChartMapPolarData, 1);
-				double minimum, maximum;
-				double z_rotation = gog_axis_get_circular_rotation (axis0) * M_PI / 180.0;
-				double perimeter;
-	
-				map->axis_map[0] = gog_axis_map_new (axis0, 0.0, 1.0);
-				gog_axis_map_get_bounds (map->axis_map[0], &minimum, &maximum);
-				if (gog_axis_is_discrete (axis0)) {
-					data->th0 = go_rint (minimum);
-					data->th1 = go_rint (maximum);
-					calc_polygon_parameters (area, data, fill_area);
-					gog_axis_map_free (map->axis_map[0]);
-					map->axis_map[0] = gog_axis_map_new (axis0, 
-						- M_PI / 2.0 + z_rotation,
-						2.0 * M_PI * (maximum - minimum) / (maximum - minimum + 1));
-				} else {
-					perimeter = gog_axis_get_polar_perimeter (axis0);
-					minimum = minimum * 2.0 * M_PI / perimeter + z_rotation;
-					maximum = maximum * 2.0 * M_PI / perimeter + z_rotation;
-					data->th0 = minimum;
-					data->th1 = maximum;
-					calc_circle_parameters (area, data, fill_area);
-					gog_axis_map_free (map->axis_map[0]);
-					map->axis_map[0] = gog_axis_map_new (axis0, -minimum,
-									     minimum - maximum);
-				}
-				map->axis_map[1] = gog_axis_map_new (axis1, 0.0, 1.0);
-				map->axis_map[2] = NULL;
-
-				map->data = data;
-				map->map_2D_to_view = polar_map_2D_to_view;
-
-				map->is_valid = gog_axis_map_is_valid (map->axis_map[0]) &&
-					gog_axis_map_is_valid (map->axis_map[1]);
-				break;
-			}
-		case GOG_AXIS_SET_XYZ:
-		case GOG_AXIS_SET_ALL:
-		case GOG_AXIS_SET_NONE:
-		case GOG_AXIS_SET_UNKNOWN:
-		default:
-			g_warning ("[Chart::map_new] not implemented for this axis set (%i)",
-				   axis_set);
-			map->map_2D_to_view = null_map_2D;
-			break;
-	}
-
-	return map;
-}
-
-/**
- * gog_chart_map_2D_to_view:
- * @map: a #GogChartMap
- * @x: data x value
- * @y: data y value
- * @u: placeholder for x converted value
- * @v: placeholder for y converted value
- *
- * Converts a 2D coordinate from data space to canvas space.
- **/
-
-void
-gog_chart_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	(map->map_2D_to_view) (map, x, y, u, v);
-}
-
-/**
- * gog_chart_map_get_axis_map:
- * @map: a #GogChartMap
- * @index: axis index
- *
- * Convenience function which returns one of the associated axis_map.
- *
- * Valid values are in range [0..2].
- *
- * returns: a #GogAxisMap.
- **/
-
-GogAxisMap *
-gog_chart_map_get_axis_map (GogChartMap *map, unsigned int i)
-{
-	g_return_val_if_fail (map != NULL, NULL);
-	g_return_val_if_fail (i < 3, NULL);
-
-	return map->axis_map[i];
-}
-
-/**
- * gog_chart_map_is_valid:
- * @map: a #GogChartMap
- *
- * Tests if @map was correctly initializied, i.e. if all associated axis_map
- * are valid (see gog_axis_map_is_valid() ).
- *
- * given 
- * to gog_chart_map_new. 
- * returns: %TRUE if @map is valid.
- **/
-
-gboolean
-gog_chart_map_is_valid (GogChartMap *map)
-{
-	g_return_val_if_fail (map != NULL, FALSE);
-
-	return map->is_valid;
-}
-
-/**
- * gog_chart_map_free:
- * @map: a #GogChartMap
- *
- * Frees @map object.
- **/
-
-void
-gog_chart_map_free (GogChartMap *map)
-{
-	int i;
-
-	g_return_if_fail (map != NULL);
-
-	for (i = 0; i < 3; i++)
-		if (map->axis_map[i] != NULL)
-			gog_axis_map_free (map->axis_map[i]);
-
-	g_free (map->data);
-	g_object_unref (map->chart);
-	g_free (map);
-}
-
 enum {
 	CHART_PROP_0,
 	CHART_PROP_CARDINALITY_VALID,
@@ -693,7 +353,7 @@ static gboolean color_axis_can_add (GogObject const *parent) { return axis_can_a
 static void color_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_COLOR); }
 
 static GogObjectRole const roles[] = {
-	{ N_("Grid"), "GogGrid",	0,
+	{ N_("Backplane"), "GogGrid",	0,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  role_grid_can_add, NULL, NULL, role_grid_post_add, role_grid_pre_remove, NULL, { -1 } },
 	{ N_("X-Axis"), "GogAxis",	1,
